@@ -43,15 +43,9 @@ describe Parallel do
       end
     end
 
-    if RUBY_VERSION.to_f >= 2.2
-      it 'uses Etc.nprocessors in Ruby 2.2+' do
-        defined?(Etc).should == "constant" 
-        Etc.respond_to?(:nprocessors).should == true 
-      end
-    else
-      it 'doesnt use Etc.nprocessors in Ruby 2.1 and below' do
-        defined?(Etc).should == "constant"
-      end
+    it 'uses Etc.nprocessors in Ruby 2.2+' do
+      defined?(Etc).should == "constant"
+      Etc.respond_to?(:nprocessors).should == true
     end
   end
 
@@ -82,10 +76,18 @@ describe Parallel do
       `ruby spec/cases/parallel_with_nil_uses_detected_cpus.rb`.should == "HELLO\n" * cpus
     end
 
+    it "executes with cpus from ENV" do
+      `PARALLEL_PROCESSOR_COUNT=10 ruby spec/cases/parallel_with_detected_cpus.rb`.should == "HELLO\n" * 10
+    end
+
     it "set amount of parallel processes" do
       `ruby spec/cases/parallel_with_set_processes.rb`.should == "HELLO\n" * 5
     end
 
+    it "enforces only one worker type" do
+      lambda { Parallel.map([1,2,3], in_processes: 2, in_threads: 3) }.should raise_error(ArgumentError)
+    end
+
     it "does not influence outside data" do
       `ruby spec/cases/parallel_influence_outside_data.rb`.should == "yes"
     end
@@ -184,7 +186,7 @@ describe Parallel do
 
     it 'does not leave processes behind while running' do
       skip if ENV['TRAVIS'] # this randomly fails on travis all the time :(
-      `ruby spec/cases/closes_processes_at_runtime.rb`.should == 'OK'
+      `ruby spec/cases/closes_processes_at_runtime.rb`.gsub(/.* deprecated; use BigDecimal.*\n/, '').should == 'OK'
     end
 
     it "does not open unnecessary pipes" do
@@ -221,6 +223,10 @@ describe Parallel do
       }.should <= 3.5
     end
 
+    it "does not modify options" do
+      lambda { Parallel.map([], {}.freeze) }.should_not raise_error
+    end
+
     it "executes with given parameters" do
       `ruby spec/cases/parallel_map.rb`.should == "-a- -b- -c- -d-"
     end
@@ -229,7 +235,7 @@ describe Parallel do
       `ruby spec/cases/parallel_map_complex_objects.rb`.should == "YES"
     end
 
-    it "starts new process imediatly when old exists" do
+    it "starts new process immediately when old exists" do
       time_taken{
       `ruby spec/cases/parallel_map_uneven.rb`
       }.should <= 3.5
@@ -379,13 +385,62 @@ describe Parallel do
       `ruby spec/cases/exit_in_process.rb 2>&1`.should include "Yep, DEAD"
     end
 
+    it "should rescue the Exception raised in child process" do
+      `ruby spec/cases/exception_raised_in_process.rb 2>&1`.should include "Yep, rescued the exception"
+    end
+
     it 'raises EOF (not DeadWorker) when a worker raises EOF in process' do
       `ruby spec/cases/eof_in_process.rb 2>&1`.should include 'Yep, EOF'
     end
 
-    it "can be killed instantly" do
-      result = `ruby spec/cases/parallel_kill.rb 2>&1`
-      result.should == "DEAD\nWorks nil\n"
+    it "threads can be killed instantly" do
+      mutex = Mutex.new
+      state = [nil, nil]
+      children = [nil, nil]
+      thread = Thread.new do
+        parent = Thread.current
+        Parallel.map([0,1], :in_threads => 2) do |i|
+          mutex.synchronize { children[i] = Thread.current }
+          mutex.synchronize { state[i] = :ready }
+          parent.join
+          mutex.synchronize { state[i] = :error }
+        end
+      end
+      while state.any? { |s| s.nil? }
+        Thread.pass
+      end
+      thread.kill
+      while children.any? { |c| c.alive? }
+        Thread.pass
+      end
+      state[0].should == :ready
+      state[1].should == :ready
+    end
+
+    it "processes can be killed instantly" do
+      pipes = [IO.pipe, IO.pipe]
+      thread = Thread.new do
+        Parallel.map([0, 1, 2, 3], :in_processes => 2) do |i|
+          pipes[i%2][0].close unless pipes[i%2][0].closed?
+          Marshal.dump('finish', pipes[i%2][1])
+          sleep 1
+          nil
+        end
+      end
+      [0, 1].each do |i|
+        Marshal.load(pipes[i][0]).should == 'finish'
+      end
+      pipes.each { |pipe| pipe[1].close }
+      thread.kill
+      pipes.each do |pipe|
+        begin
+          ret = Marshal.load(pipe[0])
+        rescue EOFError
+          ret = :error
+        end
+        ret.should == :error
+      end
+      pipes.each { |pipe| pipe[0].close }
     end
 
     it "synchronizes :start and :finish" do
@@ -403,7 +458,6 @@ describe Parallel do
     end
 
     it 'can work in isolation' do
-      skip if ENV['TRAVIS'] # this randomly hangs on travis
       out = `ruby spec/cases/map_isolation.rb`
       out.should == "1\n2\n3\n4\nOK"
     end
@@ -435,6 +489,12 @@ describe Parallel do
     end
   end
 
+  describe ".map_with_index" do
+    it "yields object and index" do
+      `ruby spec/cases/flat_map.rb 2>&1`.should == '["a", ["a"], "b", ["b"]]'
+    end
+  end
+
   describe ".any?" do
     it "returns true if any result is truthy" do
       `ruby spec/cases/any_true.rb`.split(',').should == ['true'] * 3 * 2
@@ -490,7 +550,7 @@ describe Parallel do
 
     worker_types.each do |type|
       it "works with SQLite in #{type}" do
-        `WORKER_TYPE=#{type} ruby spec/cases/each_with_ar_sqlite.rb 2>&1`.should == "Parent: X\nParallel (in_#{type}): XXX\nParent: X\n"
+        `WORKER_TYPE=#{type} ruby spec/cases/each_with_ar_sqlite.rb 2>&1`.gsub(/.* deprecated; use BigDecimal.*\n/, '').should == "Parent: X\nParallel (in_#{type}): XXX\nParent: X\n"
       end
 
       it "stops all workers when one fails in #{type}" do
@@ -590,7 +650,10 @@ describe Parallel do
 
   describe "GC" do
     def normalize(result)
-      result.sub(/\{(.*)\}/, "\\1").split(", ").reject { |x| x =~ /^(Hash|Array|String)=>(1|-1|-2)$/ }
+      result = result.sub(/\{(.*)\}/, "\\1").split(", ")
+      result.reject! { |x| x =~ /^(Hash|Array|String)=>(1|-1|-2)$/ }
+      result.reject! { |x| x =~ /^(Mutex)=>(1)$/ } if RUBY_VERSION < "2.0"
+      result
     end
 
     worker_types.each do |type|
